'use strict';

const coAsyncLib = require('async');
const coFs = require('fs');

const coPathLib = require('../paths.js');

const Result = require('./result.js').Result;
const coValidateLib = require('./validate.js');
const coDwfAbstract = require('./abstract.js');

const coAbstractLib = require('./object/type/abstract.js');
const coTxtLib = require('./object/type/txt.js');
const coNamespaceLib = require('./object/type/nameSpace.js');
const coTaskchainLib = require('./object/type/taskChain.js');
const coCrypto = require('crypto');
const coUtils = require('../utils.js');
const coPathsLib = require('../paths.js');
const coLoggerLib = require('../logger.js');


const csNamespace = 'namespace';
const csTaskChainId = 'taskChainId';

const cfGetHash = function (ioTaskChain4Batch) {
  let lsText = JSON.stringify(ioTaskChain4Batch);
  let lsHash = coCrypto.createHash('sha256').update(lsText, coUtils.encoding).digest('hex');
  return lsHash;
};
const cfGetTaskChain4BatchPost = function (isFile, ioDir2Namespace) {
  let loData_TaskChain = coUtils.readJSONFile(isFile);
  let laParsedNamelessFile = coPathsLib.parseNamelessFile(isFile);
  let lsNamespace = ioDir2Namespace[laParsedNamelessFile.dir]; // Check
  if (typeof lsNamespace === 'undefined' || lsNamespace === null) {
    throw new Error('Failed, because the \'' + '\' namespace for ' + isFile + ' file is not supported');
  }
  let lsName = laParsedNamelessFile.name;
  if (typeof lsName === 'undefined' || lsName === null) { // Check - .dwftaskchain
    throw new Error('Failed, because the \'' + '\' id for ' + isFile + ' file is not supported');
  }

  loData_TaskChain[csNamespace] = lsNamespace;
  loData_TaskChain[csTaskChainId] = lsName;

  return loData_TaskChain;
};

/*
* Receiving credentials, list of all files in the design time dir.
* iaoFilesRepo(input array which has objects in it) is an object which in fact contains response of Get call on url/repo/objects
* iaoFilesRepo [{ID, NAMESPACE, TYPE_ID(.dwftaskchain) , DESIGN_TIME_FILE_PATH, HASH_UTF8_SHA256_HEX}] till today
*/
function Container(ioCredential, ioOption, iaoFile, iaoFilesRepo) {
  this.loCredential = coValidateLib(ioCredential, 'object'); // Check - ioCredential schema validation
  this.loOption = coValidateLib(ioOption, 'object');
  this.laoFileDesign = coValidateLib(iaoFile, 'object');
  this.laoFilesRepo = coValidateLib(iaoFilesRepo, 'object');

  /*
  * it contains all the objects which have to be deployed(post) can calculated by calling function cfPostBatchFiles()
  * curently it is used only for post on taskchain endpoint url
  */
  this.laoPostBatchFiles = [];
  /*
  * it contains all objects which needs to be deleted/post from url/repo endpoint.
  * Moreover, Deletion file list for taskchain is also got from same variable
  */
  this.loResult = new Result();

  let lnVersion = ioCredential[coDwfAbstract.cs_Credential_Key_Version];
  let lsVersionErrorMessage =
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Version + ' = ' + lnVersion + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module';
  coValidateLib(lnVersion, 'number', lsVersionErrorMessage);// Check - version
  if (lnVersion < 1) { // Check
    throw new Error(lsVersionErrorMessage);
  }

  let laoProvidersV1 = ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1];
  coValidateLib(laoProvidersV1, 'object', // Check - providers_v1
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Providers_v1 + ' = ' + laoProvidersV1 + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module');

  let loRepositoryV1 = ioCredential[coDwfAbstract.cs_Credential_Key_Repository_v1];
  coValidateLib(loRepositoryV1, 'object', // Check - repository_v1
    'Failed, because the used version of the dwf-deploy does not support the current environment of the dwf module. ' +
    'The VCAP_SERVICES.dwf[<n>].credentials.' + coDwfAbstract.cs_Credential_Key_Repository_v1 + ' = ' + loRepositoryV1 + ' is not supported. ' +
    'Solution: 1). Use a different version of the dwf-deploy module 2). Update the environment of the dwf module');
  let that = this;
  let lo4AllFiles_sPathsHash = {}; // local object(map) for new files and their calculated hash;
  let lo4RepoFiles_sPathsHash = {}; // local object(map) for already deployed files and their hash;
  /*
  * It is used for new files in the design time dir
  * creates a key value pair for each file with key as its 'design time file path' and value as its hash
  */
  function lf_calculateHash4AllFiles(io_sExt2oValue) {
    let lsExt_Namespace = coNamespaceLib[coAbstractLib.csExtention];
    let loPreparedValue4Ext = io_sExt2oValue[lsExt_Namespace] || {};
    let taskChainFiles = [];
    let lasExts = ['.dwftaskchain'];
    lasExts.forEach(function (extention) {
      taskChainFiles = taskChainFiles.concat(that.o_sExt2asFile[extention]);
    });
    taskChainFiles.forEach(function (lsFile) {
      //let lsFile = file[0];
      if (coFs.statSync(lsFile).isDirectory()) { // Remove directories
      } else {
        let data = cfGetTaskChain4BatchPost(lsFile, loPreparedValue4Ext);
        lo4AllFiles_sPathsHash[lsFile] = cfGetHash(data);
      }
    });
  }
  /*
  * It is used for already deployed files
  * creates a key value pair for each file with key as its 'design time file path' and value as its hash
  */
  function lf_saveHash4DeployedFiles() {
    that.laoFilesRepo.forEach(function (currentFile) {
      lo4RepoFiles_sPathsHash[currentFile.DESIGN_TIME_FILE_PATH] = currentFile.HASH_UTF8_SHA256_HEX;
    });
  }

  lf_saveHash4DeployedFiles();
  // Supported extensions per prioroty
  this.asExt = [
    coTxtLib[coAbstractLib.csExtention],
    coNamespaceLib[coAbstractLib.csExtention],
    coTaskchainLib[coAbstractLib.csExtention]
  ];
  // Supported extention to Type and checked uniqueness
  this.o_sExt2oType = {};
  let lfPush_o_sExt2oType = function (ioType) {
    let lsExt = ioType[coAbstractLib.csExtention];
    let loType = that.o_sExt2oType[lsExt];
    if (typeof loType === 'undefined') {
      that.o_sExt2oType[lsExt] = ioType;
    } else {
      throw new Error('Failed, because the ' + lsExt + ' extention is already supported by ' +
        JSON.stringify(loType) + ' and cannot be supported also by ' + JSON.stringify(ioType));
    }
  };
  lfPush_o_sExt2oType(coTxtLib);
  lfPush_o_sExt2oType(coNamespaceLib);
  lfPush_o_sExt2oType(coTaskchainLib);

  // Group the files in the module per extention
  this.o_sExt2asFile = {};
  let lfPush_o_sExt2asFile = function (isExt, isFile) {
    let lasFile_Old = that.o_sExt2asFile[isExt];
    if (typeof lasFile_Old === 'undefined') {
      lasFile_Old = [];
    }
    let lasFile_New = lasFile_Old.concat([isFile]);
    that.o_sExt2asFile[isExt] = lasFile_New;
  };
  /*
  * This function takes isEXt as extention and isFile as filePah
  * Deletes isFile with extention isExt from  laoFilesRepo  if file is found
  * At the end any file remained in laoFilesRepo mean that file has to be delted from runtime.
  */
  function lf_UpdateFile(isExt, isFile) {
    let lasFile_Old = that.o_sExt2asFile[isExt];
    if (typeof lasFile_Old === 'undefined') {
      return;
    }
    lasFile_Old.splice(lasFile_Old.indexOf(isFile), 1);   // removing first from o_sExt2asFile which is base container for files
    that.o_sExt2asFile[isExt] = lasFile_Old;

    //removing from array that we created from Repo/objects get call
    that.laoFilesRepo.forEach(function (obj) {
      if (obj.DESIGN_TIME_FILE_PATH === isFile) {
        that.laoFilesRepo.splice(that.laoFilesRepo.indexOf(obj), 1);
      }
    });
  }
  this.laoFileDesign.forEach(function (isFile) {
    let lsFile = isFile[0];
    if (coFs.statSync(lsFile).isDirectory()) { // Remove directories
      return;
    }
    let sExtnameLowerCase = coPathLib.extnameNamelessFile(lsFile).toLowerCase(); // Group files
    lfPush_o_sExt2asFile(sExtnameLowerCase, lsFile);

  });

  // Check for unsupported extensions
  let la_sFileUnsupported = [];
  Object.getOwnPropertyNames(this.o_sExt2asFile).forEach(function (isExt) {
    let laoFilePerExt = that.o_sExt2asFile[isExt];
    if (typeof that.o_sExt2oType[isExt] === 'undefined') { // Unsupported
      la_sFileUnsupported = la_sFileUnsupported.concat(laoFilePerExt);
    }
  });
  if (la_sFileUnsupported.length > 0) {
    throw new Error('Failed, because the ' + JSON.stringify(la_sFileUnsupported) + ' files in the module are not supported');
  }
  this.loResultPrepare = this.prepare();
  if (this.loResultPrepare[Result.csKey_nStatus] === Result.cnSuccess) {

    let loResultPrepare_o_sExt2oValue = this.loResultPrepare[Result.csKey_o_sExt2oValue];
    lf_calculateHash4AllFiles(loResultPrepare_o_sExt2oValue);
    /*
    * loop on deployedFiles map to check if it matches with new files(design time)
    * on every match it calls lf_UpdateFile to delete from the correcsponding map object
    */
    let updateFiles = function () {
      Object.keys(lo4RepoFiles_sPathsHash).forEach(function (key) {
        if (lo4AllFiles_sPathsHash.hasOwnProperty(key)) {
          if (lo4RepoFiles_sPathsHash[key] === lo4AllFiles_sPathsHash[key]) {
            lf_UpdateFile('.dwftaskchain', key);
          }
        }
      });
    };
    updateFiles();
    this.laoPostBatchFiles = this.cfPostBatchFiles(loResultPrepare_o_sExt2oValue);
  }
}

Container.prototype.cfPostBatchFiles = function (io_sExt2oValue) {
  let lasFile;
  let that = this;
  let lsFunction = coAbstractLib.cfCreateTasks;
  let lsExt_Namespace = coNamespaceLib[coAbstractLib.csExtention];
  let loPreparedValue4Ext = io_sExt2oValue[lsExt_Namespace] || {};
  this.asExt.forEach(function (isExt) {
    let loType = that.o_sExt2oType[isExt];
    lasFile = that.o_sExt2asFile[isExt] || [];
    let lfRef = loType[lsFunction];
    if (typeof lfRef !== 'undefined') {
      //console.log(lasFile);
    }
  });
  let iasFile = lasFile;
  const coType = {};
  coType[coAbstractLib.csExtention] = '.dwftaskchain';
  //let loResult = new Result();
  let lsExt = coType[coAbstractLib.csExtention];
  let lo_sFile2oTaskChainBatchPost = {};
  iasFile.forEach(function (isFile) {
    let lsFile = isFile;

    coLoggerLib.trace(' * ' + lsExt + ' - ' + lsFile + ' ... ');

    let loTaskChain4Batch = cfGetTaskChain4BatchPost(lsFile, loPreparedValue4Ext);
    lo_sFile2oTaskChainBatchPost[isFile] = loTaskChain4Batch;

    let loItem = {};
    loItem[Result.csKey_AOI_Action] = Result.ACTIONS.POST;
    loItem[Result.csKey_AOI_nStatus] = Result.cnSuccess;
    loItem[Result.csKey_AOI_sFile] = lsFile;
    loItem[Result.csKey_AOI_sHashUtf8Sha256Hex] = cfGetHash(loTaskChain4Batch);
    loItem[Result.csKey_AOI_sNamespace] = loTaskChain4Batch[csNamespace];
    loItem[Result.csKey_AOI_nID] = loTaskChain4Batch[csTaskChainId];
    loItem[Result.csKey_AOI_TYPE_ID] = lsExt;
    loItem[Result.csKey_AOI_sDescription] = '';
    that.loResult.addItem(lsExt, loItem);

    coLoggerLib.log(' * ' + lsExt + ' - ' + lsFile + ' ... ok to Post');
  });
  let loTableColumns = (Result.coExtentions.hasOwnProperty(lsExt)) ? Result.coExtentions[lsExt] : {};
  let loItem = {};
  /**
   * loops over all files which are filtered after comparison with new files
   * creates an object with all info of file and set the flaf of Action as Delete
   * finally add each object in the array
   */
  that.laoFilesRepo.forEach(function (isFile) {
    let lsFile = isFile;
    loItem[Result.csKey_AOI_Action] = Result.ACTIONS.DELETE; // introducing new object property with POST/DELETE ACTION VALUE
    loItem[Result.csKey_AOI_nStatus] = Result.cnSuccess;
    loItem[Result.csKey_AOI_sDescription] = '';
    Object.keys(loTableColumns).forEach(function (key) {
      loItem[loTableColumns[key]] =  lsFile[key];
    });
    coLoggerLib.log(' * ' + lsExt + ' - ' + lsFile.DESIGN_TIME_FILE_PATH + ' ... ok to Delete');
    that.loResult.addItem(lsExt, loItem);
  });

  // Check (for two or more designtime files defining the same runtime object) and prepare
  let laoTaskChainPostBatch = [];
  let lo_sNamespace2o_sId2sFile = {}; // all for the check
  let lo_sNamespace2o_sId2asFile = {}; // errors
  iasFile.forEach(function (isFile) {
    let lsFile = isFile;

    let loTaskChainBatchPost = lo_sFile2oTaskChainBatchPost[lsFile];
    // Check
    let lsNamespace = loTaskChainBatchPost[csNamespace];
    let lsTaskChainId = loTaskChainBatchPost[csTaskChainId];
    let lo_sId2sFile = lo_sNamespace2o_sId2sFile[lsNamespace];
    if (typeof lo_sId2sFile === 'object') {
      let lsFileFirst = lo_sId2sFile[lsTaskChainId];
      if (typeof lsFileFirst === 'string') {
        if (typeof lo_sNamespace2o_sId2asFile[lsNamespace] === 'undefined') {
          lo_sNamespace2o_sId2asFile[lsNamespace] = {};
        }
        if (typeof lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId] === 'undefined') {
          lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId] = [lsFileFirst, lsFile];
        } else {
          lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId].push(lsFile);
        }
      } else {
        lo_sId2sFile[lsTaskChainId] = lsFile;
      }
    } else {
      lo_sNamespace2o_sId2sFile[lsNamespace] = {};
      lo_sNamespace2o_sId2sFile[lsNamespace][lsTaskChainId] = lsFile;
    }
    // Prepare
    laoTaskChainPostBatch.push(loTaskChainBatchPost);
  });
  if (Object.getOwnPropertyNames(lo_sNamespace2o_sId2asFile).length > 0) {
    throw new Error('Failed, because more than one designtime file claim to define the same runtime object, because of its namespace and id. ' +
      JSON.stringify(lo_sNamespace2o_sId2asFile));
  }
  return laoTaskChainPostBatch;
};

Container.prototype.prepare = function () {
  let that = this;
  let lsFunction = coAbstractLib.cfPrepare;
  let loResult = new Result();
  this.asExt.forEach(function (isExt) {
    let loType = that.o_sExt2oType[isExt];
    let lasFile = that.o_sExt2asFile[isExt] || []; // Check - empty src folder, only .txt or .dwfnamespace ot .dwftaskchain
    let lfRef = loType[lsFunction];
    if (typeof lfRef !== 'undefined') {
      let loResultExt = lfRef.apply(that, [lasFile, that.loOption]);
      loResult.addResult(loResultExt);
    }
  });
  return loResult;
};

Container.prototype.createTasks = function () {
  let that = this;
  let lsFunction = coAbstractLib.cfCreateTasks;
  let loResult = new Result();
  this.asExt.forEach(function (isExt) {
    let loType = that.o_sExt2oType[isExt];
    let lfRef = loType[lsFunction];
    if (typeof lfRef !== 'undefined') {
      let loResultExt = lfRef.apply(that, [that.laoPostBatchFiles, that.loResult, that.loCredential, that.loOption]);
      loResult.addResult(loResultExt);
    }
  });
  return loResult;
};

Container.prototype.runTasks = function (io_sExt2afTask, ifCb) {
  let lafTasks = [];

  this.asExt.forEach(function (isExt) {
    let lafTasks4Ext = io_sExt2afTask[isExt];
    if (typeof lafTasks4Ext !== 'undefined') {
      lafTasks = lafTasks.concat(lafTasks4Ext);
    }
  });

  coAsyncLib.series(lafTasks, ifCb);
};

module.exports.Container = Container;

