'use strict';

const coAsyncLib = require('async');
const coLoggerLib = require('../../../logger.js');
const coAbstractLib = require('./abstract.js');
const coRequestLib = require('../../request.js');
const Result = require('../../result.js').Result;
const coValidateLib = require('../../validate.js');
const coRepositoryLib = require('../../repository.js');
const coDwfAbstract = require('../../abstract.js');

const cfGetProvider = function (aoProviders, isExtensionFile) {
  let loProviderExt = {};
  aoProviders.forEach(function (ioProvider) { // Check - ioProvider schema validation
    let laoExtentions = ioProvider.extensions;
    laoExtentions.forEach(function(ioExtension) {
      let lsExtensionName = ioExtension.name;
      let lsExtensionRoute = ioExtension.route;
      if (isExtensionFile === '.' + lsExtensionName) {
        loProviderExt.org = ioProvider.org;
        loProviderExt.org_guid = ioProvider.org_guid;
        loProviderExt.space = ioProvider.space;
        loProviderExt.space_guid = ioProvider.space_guid;
        loProviderExt.name = ioProvider.name;
        loProviderExt.url = ioProvider.url + '/' + lsExtensionRoute;
        loProviderExt.extention = lsExtensionName;
        loProviderExt.route = lsExtensionRoute;
      }
    });
  });
  if (typeof loProviderExt.url === 'undefined' || loProviderExt.url === null) {
    throw new Error('Failed, because was not able to construct the url for the service provider for ' + isExtensionFile + ' files');
  }
  return loProviderExt;
};
const cfTaskChainPostBatch = function(isExt, iaoTaskChainPostBatch, iaoResultItem, ioProviderExt, ioCredential) {

  let laoObjectInstanceToDelete = [];
  let laoResultItem = iaoResultItem.get_aoItem(isExt);
  laoResultItem.forEach(function(ioResultItem) {
    if(ioResultItem[Result.csKey_AOI_Action] === Result.ACTIONS.DELETE) {
        let loObjectInstance = {
      'namespace': ioResultItem[Result.csKey_AOI_sNamespace],
      'taskChainId': ioResultItem[Result.csKey_AOI_nID],
      'clear': true
    };
    laoObjectInstanceToDelete.push(loObjectInstance);
    }
  });

  let lsProviderUrl = ioProviderExt.url;
  let loData = {
    uri: lsProviderUrl,
    json: true,
    method: 'POST',
    auth: {
      user: ioCredential.user,
      password: ioCredential.password,
      sendImmediately: true
    },
    body: iaoTaskChainPostBatch
  };
  let lfTask = function (ifCb) {
    coLoggerLib.trace(' * Test ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ');
    let lfcb = coRepositoryLib.lfCb2Reuse(isExt, loData, ifCb );
    coRequestLib(loData, lfcb);
  };
  let lafTask = [];
  lafTask.push(lfTask);
    let loData4Delete = {
    uri: lsProviderUrl,
    json: true,
    method: 'DELETE',
    auth: {
      user: ioCredential.user,
      password: ioCredential.password,
      sendImmediately: true
    },
    body: laoObjectInstanceToDelete
  };
  let lfTaskDelete = function (ifCb) {
    coLoggerLib.trace(' * ' + isExt + ' - ' + loData4Delete.uri + ' ' + loData4Delete.method + ' ... ');
    let lfcb = coRepositoryLib.lfCb2Reuse(isExt, loData4Delete, ifCb );
    coRequestLib(loData4Delete, lfcb);
  };
   lafTask.push(lfTaskDelete);
  return lafTask;
};

const cfTaskChainPostRepo = function(isExt, iaoResultItem, ioProviderExt, ioCredential, ioOption) {
  let loRepository = new coRepositoryLib.Repository(ioCredential, ioOption);
  let lafTaskWaterfall = loRepository.get_afTask2Update(isExt, iaoResultItem, ioProviderExt);
  let lafTask = [];
  lafTask.push(function(ifCb) {
    coAsyncLib.waterfall(lafTaskWaterfall, function(ioError, ioResult) {
      ifCb(ioError, ioResult);
    });
  });
  return lafTask;
};

const coType = {};
coType[coAbstractLib.csExtention] = '.dwftaskchain';
coType[coAbstractLib.cfPrepare] = undefined;
coType[coAbstractLib.cfCreateTasks] = function(iaoTaskChainPostBatch, ioResult, ioCredential, ioOption) {
  coValidateLib(iaoTaskChainPostBatch.length, 'number');
  coValidateLib(ioResult, 'object');
  coValidateLib(ioCredential, 'object');
  coValidateLib(ioOption, 'object');

  let loResult = ioResult;
  let lsExt = coType[coAbstractLib.csExtention];

  // Creates the Task Chain for the Batch Post
  let laoTaskChainPostBatch = iaoTaskChainPostBatch;//coContainerLib.cfCalculateHash(iasFile, ioPreparedValue4Ext, ioCredential, ioOption);
  // Greates tasks for TOE and Repo
  let loProviderExt = cfGetProvider(ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1], lsExt);
  let loValue_afTask = [];
  let lafTaskToe = cfTaskChainPostBatch(lsExt, laoTaskChainPostBatch, loResult, loProviderExt, ioCredential);
  loValue_afTask = loValue_afTask.concat(lafTaskToe);
  let lafTaskRepo = cfTaskChainPostRepo(lsExt, loResult.get_aoItem(lsExt), loProviderExt, ioCredential, ioOption);
  loValue_afTask = loValue_afTask.concat(lafTaskRepo);
  loResult.addValue(lsExt, loValue_afTask);

  return loResult;
};

module.exports = coType;
