'use strict';

const coValidateLib = require('./validate.js');

function Result() {
  coValidateLib(arguments.length === 0, 'boolean');

  this[Result.csKey_nStatus] = coValidateLib(Result.cnSuccess, 'number');
  this[Result.csKey_sDescription] = '';
  this[Result.csKey_o_sExt2aoItem] = {};
  this[Result.csKey_o_sExt2oValue] = {};
}

Result.csKey_nStatus = 'nStatus';
Result.csKey_sDescription = 'sDescription';
Result.csKey_o_sExt2aoItem = 'o_sExt2aoItem';
Result.csKey_o_sExt2oValue = 'o_sExt2oValue';

Result.csKey_AOI_nStatus = 'nStatus';
Result.csKey_AOI_sFile = 'sFile';
Result.csKey_AOI_sHashUtf8Sha256Hex = 'sHashUtf8Sha256Hex';
Result.csKey_AOI_sNamespace = 'sNamespace';
Result.csKey_AOI_nID = 'sID';
Result.csKey_AOI_sDescription = 'sDescription';
Result.csKey_AOI_Action = 'sAction';
Result.csKey_AOI_TYPE_ID = 'TYPE_ID';
Result.ACTIONS = {'GET':'GET', 'POST': 'POST','DELETE': 'DELETE'};
Result.cnSuccess = 0;
Result.cnWarning = 1;
Result.cnError = 2;
Result.coExtentions = {
  '.dwftaskchain': {
    'NAMESPACE': Result.csKey_AOI_sNamespace,
    'ID': Result.csKey_AOI_nID,
    'TYPE_ID': Result.csKey_AOI_TYPE_ID,
    'DESIGN_TIME_FILE_PATH': Result.csKey_AOI_sFile,
    'HASH_UTF8_SHA256_HEX': Result.csKey_AOI_sHashUtf8Sha256Hex
  }
};


Result.prototype.get_aoItem = function(isExt) {
  let laoItem = this[Result.csKey_o_sExt2aoItem][isExt];
  if (typeof laoItem === 'undefined') {
    this[Result.csKey_o_sExt2aoItem][isExt] = [];
  }
  laoItem = this[Result.csKey_o_sExt2aoItem][isExt];
  return laoItem;
};

Result.prototype.addItem = function(isExt, ioItem) {
  coValidateLib(isExt, 'string');
  coValidateLib(ioItem, 'object');

  let lnStatus = coValidateLib(ioItem[Result.csKey_AOI_nStatus], 'number');
  coValidateLib(ioItem[Result.csKey_AOI_sFile], 'string');
  coValidateLib(ioItem[Result.csKey_AOI_sDescription], 'string');

  if (this[Result.csKey_nStatus] < lnStatus) {
    this[Result.csKey_nStatus] = lnStatus;
  }
  let laoItem = this[Result.csKey_o_sExt2aoItem][isExt];
  if (typeof laoItem === 'undefined') {
    laoItem = [];
  }
  laoItem.push(ioItem);
  this[Result.csKey_o_sExt2aoItem][isExt] = laoItem;
};

Result.prototype.addValue = function(isExt, ioValue) {
  coValidateLib(isExt, 'string');
  coValidateLib(ioValue, 'object');

  let loValue = this[Result.csKey_o_sExt2oValue][isExt];
  if (typeof loValue !== 'undefined') {
      throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
                      JSON.stringify(loValue) + ' and cannot be mapped again to ' + JSON.stringify(ioValue));
  }
  this[Result.csKey_o_sExt2oValue][isExt] = ioValue;
};

Result.prototype.addResult = function(ioResult) {
  coValidateLib(ioResult, 'object');

  let that = this;
  if (this[Result.csKey_nStatus] < ioResult[Result.csKey_nStatus]) {
    this[Result.csKey_nStatus] = ioResult[Result.csKey_nStatus];
    this[Result.csKey_sDescription] = ioResult[Result.csKey_sDescription];
  }
  Object.getOwnPropertyNames(ioResult[Result.csKey_o_sExt2aoItem]).forEach(function(isExt) {
    let laoItemDelta = ioResult[Result.csKey_o_sExt2aoItem][isExt];
    let laoItem = that[Result.csKey_o_sExt2aoItem][isExt];
    if (typeof laoItem !== 'undefined') {
      throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
                      JSON.stringify(laoItem) + ' and cannot be mapped again to ' + JSON.stringify(laoItemDelta));
    }
    that[Result.csKey_o_sExt2aoItem][isExt] = laoItemDelta;
  });
  Object.getOwnPropertyNames(ioResult[Result.csKey_o_sExt2oValue]).forEach(function(isExt) {
    let loValueDelta = ioResult[Result.csKey_o_sExt2oValue][isExt];
    let loValue = that[Result.csKey_o_sExt2oValue][isExt];

    if (typeof loValue !== 'undefined') {
      throw new Error('Failed, because the ' + isExt + ' extention is already mapped to ' +
                      JSON.stringify(loValue) + ' and cannot be mapped again to ' + JSON.stringify(loValueDelta));
    }
    that[Result.csKey_o_sExt2oValue][isExt] = loValueDelta;
  });
};

module.exports.Result = Result;
