'use strict';
var fs = require('fs');
var paths = require('./paths.js');
var utils = require('./utils.js');
var lPath = require('path');
const validator = require('./dwf/object/validator.js');

let pathSeperator;
let baseDir;
let errorMap = new Map();
let errorStack = '';
// helper function for calculating nameSpaces and rteturning a map object;
function getNameSpacesHelper(dir, nameSpace, rfMap) {
  let localDirs = [];  // For keeping record of directories in current directory
  let fileWithNameSpace; // it is undefined as long as we don't find any .hdinamespace file in current directory
  let defualtNSObject = nameSpace;  // keeping track of current namespace to carry into sub directories

  dir = paths.clientPath(dir);
  // dir has whole path i.e  src/subdir/subsubdir this code block is just to split whole path and last directory in the path. in simple words, to get name of current directory.
  let currentDir = dir.split(pathSeperator);
  if (currentDir) {
    let cdLength = currentDir.length - 1;
    while (cdLength >= 0) {
      if (currentDir[cdLength] !== '') {
        currentDir = currentDir[cdLength];
        cdLength = 0;
      }
      cdLength--;
    }
  }
  // Reding current directory using absolute path
  var files = fs.readdirSync(dir);
  files.forEach(function (ioFile) {
    var name = dir + pathSeperator + ioFile;
    if (fs.statSync(name).isFile()) {
      var sExtnameLowerCase = paths.extnameNamelessFile(name).toLowerCase();
      if (sExtnameLowerCase === '.dwfnamespace') {
        try {
          // if file is with extention hdinamespace it calls get JsonObject to get a json object out of file.
          fileWithNameSpace = utils.readJSONFile(name);
          let val = validator.validate('dwfnamespace', fileWithNameSpace);
          if (val && val.valid) {
          } else {
            let key = lPath.normalize(name);
            errorMap.set(key, val.msg);
            errorStack = errorStack + ' \n ' + key + ':0:0 error : ' + JSON.stringify(val.msg);
          }
        } catch (e) {
          throw (e); // throw or emit?
        }
      }
    } else if (fs.statSync(name).isDirectory()) {
      // adding directories in the array for recursive call later.
      localDirs.push(ioFile);
    } else {
      console.log('else');
    }
  });
  // instead of whole path c://.... we will keep only path from directory which is received as parameter
  let directoryPath;
  if (dir === baseDir) {
    directoryPath = dir;
  } else {
    directoryPath = dir.split(baseDir);
    if (directoryPath) {
      if (directoryPath.length === 1) {
        directoryPath = directoryPath[0];
      } else if (directoryPath.length > 1 && directoryPath[directoryPath.length - 1] !== '') {
        directoryPath = directoryPath[directoryPath.length - 1];
      }
      let lastIndex = directoryPath.lastIndexOf(pathSeperator);
      if (lastIndex === directoryPath.length - 1) {
        directoryPath = directoryPath.substring(0, lastIndex);
      }
    } else {
      //need to decide
    }
  }
  directoryPath = lPath.normalize(directoryPath); // removes multiple // or .. in the path
  directoryPath = paths.clientPath(directoryPath);

  // if we find a .hdinamespace file then we get name from file and add directory name and namespace in the map.
  if (fileWithNameSpace) {
    defualtNSObject = fileWithNameSpace;
    rfMap[directoryPath] = fileWithNameSpace.name;
  } // if we don't find any .hdinamespace then we check if there is namespace object from previous dir ?
  else {
    if (nameSpace && nameSpace.subfolder) {
      //special case which comes in test cases only

      // if namespace object is there and subfolder value is append: then we use name/dirname and add it to mapobject and defualtNSObject to use it in future.
      if (nameSpace.subfolder.toLocaleLowerCase() === 'append') {
        rfMap[directoryPath] = nameSpace.name + '.' + currentDir;
      } else if (nameSpace.subfolder === 'ignore') { // if namespace subfolder value is ignore then we use only name of namespace and don't add dirname (path is absolute path to current dir)
        rfMap[directoryPath] = nameSpace.name;
      } else {
        throw new Error('Unsupported  "' + nameSpace.subfolder + '" subfolder value ');
      }
    }
  }
  // loop on  all the directories which were found in the current Directory (recursive call)
  localDirs.forEach(function (ioLocalDir) {
    getNameSpacesHelper(dir + pathSeperator + ioLocalDir, defualtNSObject, rfMap);
  });

  return rfMap;
}

function setPathSeperator() {
  //need to implement logic for windows and linux
  pathSeperator = '/';
  pathSeperator = paths.clientPath(pathSeperator);
}

function getGlobalNameSpace() {
  let mtaName;
  let moduleName;
  if (process && process.env && process.env.MTA_METADATA) {
    mtaName = JSON.parse(process.env.MTA_METADATA).id;
  } else {
    //what to  do ??
  }
  if (process && process.env && process.env.MTA_MODULE_METADATA) {
    moduleName = JSON.parse(process.env.MTA_MODULE_METADATA).name;
  } else {
    //what to do ??
  }
  let nsObject = {};
  nsObject.name = mtaName + '.' + moduleName;
  nsObject.subfolder = 'append';
  return nsObject;
}

function getNameSpaces(dirs, nameSpace) {
  let loMap = {};
  setPathSeperator();
  if (!Array.isArray(dirs)) {
    console.log('There are no directories to check');
    return loMap;
  }
  //if namespace object is undefined or null
  if (!nameSpace) {
    nameSpace = getGlobalNameSpace();
  }
  let finalDirs = [];

  dirs.forEach(function (d) {
    d = paths.clientPath(d);
    if (fs.existsSync(d)) {
      finalDirs.push(d);
    }
  });

  finalDirs.forEach(function (dir) {
    dir = paths.clientPath(dir); // it converts '\' to '/' or vice versa depending on clientPath
    baseDir = paths.dirname(dir);
    baseDir = paths.dirname(baseDir); // getting parent directory .
    if (baseDir === '.') {
      baseDir = './';
    }
    loMap = getNameSpacesHelper(dir, nameSpace, loMap);

    if (errorMap && errorMap.size > 0) {
      throw errorStack;
    } else {
      //console.log(' namespace schema validation passed  ');
    }
  });

  return loMap;

}
module.exports = {
  getNameSpaces: getNameSpaces,
  getGlobalNameSpace: getGlobalNameSpace
};
